@@ -18,17 +18,25 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
-
-#ifndef GST_DISABLE_GST_DEBUG
+/* TODO:
+ * edge [ constraint=false ];
+ * edge [ minlen=0 ];
+ *   does not create spacial dependency
+ * node [ margin="0.02,0.01" ];
+ *   space surrounding the label
+ */
 
 #include "gst_private.h"
+#include "gstdebugutils.h"
+
+#ifndef GST_DISABLE_GST_DEBUG
 
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <errno.h>
 
 #include "gstinfo.h"
-#include "gstdebugutils.h"
 #include "gstbin.h"
 #include "gstobject.h"
 #include "gstghostpad.h"
@@ -453,7 +461,8 @@ _gst_debug_bin_to_dot_file (GstBin * bin, GstDebugGraphDetails details,
         "  fontname=\"Bitstream Vera Sans\";\n"
         "  fontsize=\"8\";\n"
         "  labelloc=t;\n"
-        "  nodesep=.15;\n"
+        "  nodesep=.1;\n"
+        "  ranksep=.2;\n"
         "  label=\"<%s>\\n%s%s%s\";\n"
         "  node [style=filled, shape=box, fontsize=\"7\", fontname=\"Bitstream Vera Sans\"];\n"
         "  edge [labelfontsize=\"7\", fontsize=\"7\", labelfontname=\"Bitstream Vera Sans\", fontname=\"Bitstream Vera Sans\"];\n"
@@ -470,8 +479,11 @@ _gst_debug_bin_to_dot_file (GstBin * bin, GstDebugGraphDetails details,
     /* write footer */
     fprintf (out, "}\n");
     fclose (out);
+    GST_INFO ("wrote bin graph to : '%s'", full_file_name);
+  } else {
+    GST_WARNING ("Failed to open file '%s' for writing: %s", full_file_name,
+        g_strerror (errno));
   }
-  GST_INFO ("wrote bin graph to : '%s'", full_file_name);
   g_free (full_file_name);
 }
 
@@ -488,7 +500,6 @@ _gst_debug_bin_to_dot_file_with_ts (GstBin * bin, GstDebugGraphDetails details,
     const gchar * file_name)
 {
   gchar *ts_file_name = NULL;
-  GTimeVal now;
   GstClockTime elapsed;
 
   g_return_if_fail (GST_IS_BIN (bin));
@@ -500,8 +511,8 @@ _gst_debug_bin_to_dot_file_with_ts (GstBin * bin, GstDebugGraphDetails details,
   }
 
   /* add timestamp */
-  g_get_current_time (&now);
-  elapsed = GST_TIMEVAL_TO_TIME (now) - _priv_gst_info_start_time;
+  elapsed = GST_CLOCK_DIFF (_priv_gst_info_start_time,
+      gst_util_get_timestamp ());
   ts_file_name =
       g_strdup_printf ("%" GST_TIME_FORMAT "-%s", GST_TIME_ARGS (elapsed),
       file_name);
