@@ -57,9 +57,9 @@ gst_util_dump_mem (const guchar * mem, guint size)
   i = j = 0;
   while (i < size) {
     if (g_ascii_isprint (mem[i]))
-      g_string_append_printf (chars, "%c", mem[i]);
+      g_string_append_c (chars, mem[i]);
     else
-      g_string_append_printf (chars, ".");
+      g_string_append_c (chars, '.');
 
     g_string_append_printf (string, "%02x ", mem[i]);
 
@@ -886,7 +886,7 @@ gst_element_request_compatible_pad (GstElement * element,
  * cases.
  *
  * Returns: the #GstPad to which a link can be made, or %NULL if one cannot be
- * found.
+ * found. gst_object_unref() after usage.
  */
 GstPad *
 gst_element_get_compatible_pad (GstElement * element, GstPad * pad,
@@ -1031,6 +1031,8 @@ gst_element_state_get_name (GstState state)
  * Gets a string representing the given state change result.
  *
  * Returns: a string with the name of the state change result.
+ *
+ * Since: 0.10.11
  */
 G_CONST_RETURN gchar *
 gst_element_state_change_return_get_name (GstStateChangeReturn state_ret)
@@ -3088,7 +3090,7 @@ element_find_unconnected_pad (GstElement * element, GstPadDirection direction)
       iter = gst_element_iterate_sink_pads (element);
       break;
     default:
-      g_assert_not_reached ();
+      g_return_val_if_reached (NULL);
   }
 
   done = FALSE;
@@ -3327,3 +3329,30 @@ gst_type_register_static_full (GType parent_type,
 
   return g_type_register_static (parent_type, type_name, &info, flags);
 }
+
+
+/**
+ * gst_util_get_timestamp:
+ *
+ * Get a timestamp as GstClockTime to be used for interval meassurements.
+ * The timestamp should not be interpreted in any other way.
+ *
+ * Returns: the timestamp
+ *
+ * Since: 0.10.16
+ */
+GstClockTime
+gst_util_get_timestamp (void)
+{
+#if defined (HAVE_POSIX_TIMERS) && defined(HAVE_MONOTONIC_CLOCK)
+  struct timespec now;
+
+  clock_gettime (CLOCK_MONOTONIC, &now);
+  return GST_TIMESPEC_TO_TIME (now);
+#else
+  GTimeVal now;
+
+  g_get_current_time (&now);
+  return GST_TIMEVAL_TO_TIME (now);
+#endif
+}
