@@ -154,36 +154,21 @@
  * been processed and the pipeline can safely be stopped.
  * </para>
  * <para>
- * Since GStreamer 0.10.3 an application may simply set the source
- * element to NULL or READY state to make it send an EOS event downstream.
- * The application should lock the state of the source afterwards, so that
- * shutting down the pipeline from PLAYING doesn't temporarily start up the
- * source element for a second time:
- * <programlisting>
- * ...
- * // stop recording
- * gst_element_set_state (audio_source, #GST_STATE_NULL);
- * gst_element_set_locked_state (audio_source, %TRUE);
- * ...
- * </programlisting>
- * Now the application should wait for an EOS message
- * to be posted on the pipeline's bus. Once it has received
- * an EOS message, it may safely shut down the entire pipeline:
- * <programlisting>
- * ...
- * // everything done - shut down pipeline
- * gst_element_set_state (pipeline, #GST_STATE_NULL);
- * gst_element_set_locked_state (audio_source, %FALSE);
- * ...
- * </programlisting>
+ * Since GStreamer 0.10.16 an application may send an EOS event to a source
+ * element to make it send an EOS event downstream. This can typically be done
+ * with the gst_element_send_event() function on the element or its parent bin.
  * </para>
  * <para>
- * Note that setting the source element to NULL or READY when the 
- * pipeline is in the PAUSED state may cause a deadlock since the streaming
- * thread might be blocked in PREROLL.
+ * After the EOS has been sent to the element, the application should wait for
+ * an EOS message to be posted on the pipeline's bus. Once this EOS message is
+ * received, it may safely shut down the entire pipeline.
  * </para>
  * <para>
- * Last reviewed on 2007-09-13 (0.10.15)
+ * The old behaviour for controlled shutdown introduced since GStreamer 0.10.3
+ * is still available but deprecated as it is dangerous and less flexible.
+ * </para>
+ * <para>
+ * Last reviewed on 2007-12-19 (0.10.16)
  * </para>
  * </refsect2>
  */
@@ -249,6 +234,9 @@ struct _GstBaseSrcPrivate
   GstEvent *close_segment;
   GstEvent *start_segment;
 
+  /* if EOS is pending */
+  gboolean pending_eos;
+
   /* startup latency is the time it takes between going to PLAYING and producing
    * the first BUFFER with running_time 0. This value is included in the latency
    * reporting. */
@@ -484,17 +472,11 @@ GstFlowReturn
 gst_base_src_wait_playing (GstBaseSrc * src)
 {
   /* block until the state changes, or we get a flush, or something */
-  if (src->is_live) {
-    while (G_UNLIKELY (!src->live_running)) {
-      GST_DEBUG ("live source signal waiting");
-      GST_LIVE_SIGNAL (src);
-      GST_DEBUG ("live source waiting for running state");
-      GST_LIVE_WAIT (src);
-      GST_DEBUG ("live source unlocked");
-    }
-    if (src->priv->flushing)
-      goto flushing;
-  }
+  GST_DEBUG_OBJECT (src, "live source waiting for running state");
+  GST_LIVE_WAIT (src);
+  if (src->priv->flushing)
+    goto flushing;
+  GST_DEBUG_OBJECT (src, "live source unlocked");
 
   return GST_FLOW_OK;
 
@@ -913,11 +895,9 @@ gst_base_src_default_do_seek (GstBaseSrc * src, GstSegment * segment)
 
   /* update our offset if the start/stop position was updated */
   if (segment->format == GST_FORMAT_BYTES) {
-    segment->last_stop = segment->start;
     segment->time = segment->start;
   } else if (segment->start == 0) {
     /* seek to start, we can implement a default for this. */
-    segment->last_stop = 0;
     segment->time = 0;
     res = TRUE;
   } else
@@ -1207,7 +1187,7 @@ gst_base_src_perform_seek (GstBaseSrc * src, GstEvent * event, gboolean unlock)
               src->segment.format, src->segment.last_stop));
     }
 
-    /* for deriving a stop position for the playback segment form the seek
+    /* for deriving a stop position for the playback segment from the seek
      * segment, we must take the duration when the stop is not set */
     if ((stop = src->segment.stop) == -1)
       stop = src->segment.duration;
@@ -1219,10 +1199,19 @@ gst_base_src_perform_seek (GstBaseSrc * src, GstEvent * event, gboolean unlock)
      * next time it is scheduled. */
     if (src->priv->start_segment)
       gst_event_unref (src->priv->start_segment);
-    src->priv->start_segment =
-        gst_event_new_new_segment_full (FALSE,
-        src->segment.rate, src->segment.applied_rate, src->segment.format,
-        src->segment.last_stop, stop, src->segment.time);
+    if (src->segment.rate >= 0.0) {
+      /* forward, we send data from last_stop to stop */
+      src->priv->start_segment =
+          gst_event_new_new_segment_full (FALSE,
+          src->segment.rate, src->segment.applied_rate, src->segment.format,
+          src->segment.last_stop, stop, src->segment.time);
+    } else {
+      /* reverse, we send data from stop to last_stop */
+      src->priv->start_segment =
+          gst_event_new_new_segment_full (FALSE,
+          src->segment.rate, src->segment.applied_rate, src->segment.format,
+          src->segment.start, src->segment.last_stop, src->segment.time);
+    }
   }
 
   src->priv->discont = TRUE;
@@ -1284,8 +1273,12 @@ gst_base_src_send_event (GstElement * element, GstEvent * event)
 
       /* downstream serialized events */
     case GST_EVENT_EOS:
-      /* FIXME, queue EOS and make sure the task or pull function 
-       * perform the EOS actions. */
+      /* queue EOS and make sure the task or pull function 
+       * performs the EOS actions. */
+      GST_LIVE_LOCK (src);
+      src->priv->pending_eos = TRUE;
+      GST_LIVE_UNLOCK (src);
+      result = TRUE;
       break;
     case GST_EVENT_NEWSEGMENT:
       /* sending random NEWSEGMENT downstream can break sync. */
@@ -1751,9 +1744,13 @@ gst_base_src_get_range (GstBaseSrc * src, guint64 offset, guint length,
 
   bclass = GST_BASE_SRC_GET_CLASS (src);
 
-  ret = gst_base_src_wait_playing (src);
-  if (ret != GST_FLOW_OK)
-    goto stopped;
+  if (src->is_live) {
+    while (G_UNLIKELY (!src->live_running)) {
+      ret = gst_base_src_wait_playing (src);
+      if (ret != GST_FLOW_OK)
+        goto stopped;
+    }
+  }
 
   if (G_UNLIKELY (!GST_OBJECT_FLAG_IS_SET (src, GST_BASE_SRC_STARTED)))
     goto not_started;
@@ -1789,9 +1786,12 @@ gst_base_src_get_range (GstBaseSrc * src, guint64 offset, guint length,
   status = gst_base_src_do_sync (src, *buf);
 
   /* waiting for the clock could have made us flushing */
-  if (src->priv->flushing)
+  if (G_UNLIKELY (src->priv->flushing))
     goto flushing;
 
+  if (G_UNLIKELY (src->priv->pending_eos))
+    goto eos;
+
   switch (status) {
     case GST_CLOCK_EARLY:
       /* the buffer is too late. We currently don't drop the buffer. */
@@ -1865,6 +1865,13 @@ flushing:
     *buf = NULL;
     return GST_FLOW_WRONG_STATE;
   }
+eos:
+  {
+    GST_DEBUG_OBJECT (src, "we are EOS");
+    gst_buffer_unref (*buf);
+    *buf = NULL;
+    return GST_FLOW_UNEXPECTED;
+  }
 }
 
 static GstFlowReturn
@@ -1877,9 +1884,13 @@ gst_base_src_pad_get_range (GstPad * pad, guint64 offset, guint length,
   src = GST_BASE_SRC (gst_pad_get_parent (pad));
 
   GST_LIVE_LOCK (src);
-  if (src->priv->flushing)
+  if (G_UNLIKELY (src->priv->flushing))
     goto flushing;
 
+  /* if we're EOS, return right away */
+  if (G_UNLIKELY (src->priv->pending_eos))
+    goto eos;
+
   res = gst_base_src_get_range (src, offset, length, buf);
 
 done:
@@ -1896,6 +1907,12 @@ flushing:
     res = GST_FLOW_WRONG_STATE;
     goto done;
   }
+eos:
+  {
+    GST_DEBUG_OBJECT (src, "we are EOS");
+    res = GST_FLOW_UNEXPECTED;
+    goto done;
+  }
 }
 
 static gboolean
@@ -1948,12 +1965,10 @@ gst_base_src_pad_check_get_range (GstPad * pad)
   GstBaseSrc *src;
   gboolean res;
 
-  src = GST_BASE_SRC (gst_pad_get_parent (pad));
+  src = GST_BASE_SRC (GST_OBJECT_PARENT (pad));
 
   res = gst_base_src_check_get_range (src);
 
-  gst_object_unref (src);
-
   return res;
 }
 
@@ -1965,24 +1980,42 @@ gst_base_src_loop (GstPad * pad)
   GstFlowReturn ret;
   gint64 position;
   gboolean eos;
+  gulong blocksize;
 
   eos = FALSE;
 
-  src = GST_BASE_SRC (gst_pad_get_parent (pad));
+  src = GST_BASE_SRC (GST_OBJECT_PARENT (pad));
 
   GST_LIVE_LOCK (src);
-  if (src->priv->flushing)
+  if (G_UNLIKELY (src->priv->flushing))
     goto flushing;
 
+  /* if we're EOS, return right away */
+  if (G_UNLIKELY (src->priv->pending_eos))
+    goto eos;
+
   src->priv->last_sent_eos = FALSE;
 
+  blocksize = src->blocksize;
+
   /* if we operate in bytes, we can calculate an offset */
-  if (src->segment.format == GST_FORMAT_BYTES)
+  if (src->segment.format == GST_FORMAT_BYTES) {
     position = src->segment.last_stop;
-  else
+    /* for negative rates, start with subtracting the blocksize */
+    if (src->segment.rate < 0.0) {
+      /* we cannot go below segment.start */
+      if (position > src->segment.start + blocksize)
+        position -= blocksize;
+      else {
+        /* last block, remainder up to segment.start */
+        blocksize = position - src->segment.start;
+        position = src->segment.start;
+      }
+    }
+  } else
     position = -1;
 
-  ret = gst_base_src_get_range (src, position, src->blocksize, &buf);
+  ret = gst_base_src_get_range (src, position, blocksize, &buf);
   if (G_UNLIKELY (ret != GST_FLOW_OK)) {
     GST_INFO_OBJECT (src, "pausing after gst_base_src_get_range() = %s",
         gst_flow_get_name (ret));
@@ -1994,11 +2027,11 @@ gst_base_src_loop (GstPad * pad)
     goto null_buffer;
 
   /* push events to close/start our segment before we push the buffer. */
-  if (src->priv->close_segment) {
+  if (G_UNLIKELY (src->priv->close_segment)) {
     gst_pad_push_event (pad, src->priv->close_segment);
     src->priv->close_segment = NULL;
   }
-  if (src->priv->start_segment) {
+  if (G_UNLIKELY (src->priv->start_segment)) {
     gst_pad_push_event (pad, src->priv->start_segment);
     src->priv->start_segment = NULL;
   }
@@ -2006,8 +2039,14 @@ gst_base_src_loop (GstPad * pad)
   /* figure out the new position */
   switch (src->segment.format) {
     case GST_FORMAT_BYTES:
-      position += GST_BUFFER_SIZE (buf);
+    {
+      guint bufsize = GST_BUFFER_SIZE (buf);
+
+      /* we subtracted above for negative rates */
+      if (src->segment.rate >= 0.0)
+        position += bufsize;
       break;
+    }
     case GST_FORMAT_TIME:
     {
       GstClockTime start, duration;
@@ -2020,23 +2059,44 @@ gst_base_src_loop (GstPad * pad)
       else
         position = src->segment.last_stop;
 
-      if (GST_CLOCK_TIME_IS_VALID (duration))
-        position += duration;
+      if (GST_CLOCK_TIME_IS_VALID (duration)) {
+        if (src->segment.rate >= 0.0)
+          position += duration;
+        else if (position > duration)
+          position -= duration;
+        else
+          position = 0;
+      }
       break;
     }
     case GST_FORMAT_DEFAULT:
-      position = GST_BUFFER_OFFSET_END (buf);
+      if (src->segment.rate >= 0.0)
+        position = GST_BUFFER_OFFSET_END (buf);
+      else
+        position = GST_BUFFER_OFFSET (buf);
       break;
     default:
       position = -1;
       break;
   }
   if (position != -1) {
-    if (src->segment.stop != -1) {
-      if (position >= src->segment.stop) {
-        eos = TRUE;
-        position = src->segment.stop;
+    if (src->segment.rate >= 0.0) {
+      /* positive rate, check if we reached the stop */
+      if (src->segment.stop != -1) {
+        if (position >= src->segment.stop) {
+          eos = TRUE;
+          position = src->segment.stop;
+        }
       }
+    } else {
+      /* negative rate, check if we reached the start. start is always set to
+       * something different from -1 */
+      if (position <= src->segment.start) {
+        eos = TRUE;
+        position = src->segment.start;
+      }
+      /* when going reverse, all buffers are DISCONT */
+      src->priv->discont = TRUE;
     }
     gst_segment_set_last_stop (&src->segment, src->segment.format, position);
   }
@@ -2055,14 +2115,13 @@ gst_base_src_loop (GstPad * pad)
     goto pause;
   }
 
-  if (eos) {
-    GST_INFO_OBJECT (src, "pausing after EOS");
+  if (G_UNLIKELY (eos)) {
+    GST_INFO_OBJECT (src, "pausing after end of segment");
     ret = GST_FLOW_UNEXPECTED;
     goto pause;
   }
 
 done:
-  gst_object_unref (src);
   return;
 
   /* special cases */
@@ -2073,6 +2132,13 @@ flushing:
     ret = GST_FLOW_WRONG_STATE;
     goto pause;
   }
+eos:
+  {
+    GST_DEBUG_OBJECT (src, "we are EOS");
+    GST_LIVE_UNLOCK (src);
+    ret = GST_FLOW_UNEXPECTED;
+    goto pause;
+  }
 pause:
   {
     const gchar *reason = gst_flow_get_name (ret);
@@ -2345,6 +2411,8 @@ gst_base_src_set_flushing (GstBaseSrc * basesrc,
   if (flushing) {
     /* if we are locked in the live lock, signal it to make it flush */
     basesrc->live_running = TRUE;
+    /* clear pending EOS if any */
+    basesrc->priv->pending_eos = FALSE;
 
     /* step 1, now that we have the LIVE lock, clear our unlock request */
     if (bclass->unlock_stop)
@@ -2383,12 +2451,14 @@ gst_base_src_set_playing (GstBaseSrc * basesrc, gboolean live_play)
    * waiting for PLAYING while blocked in the LIVE cond or we can be waiting
    * for the clock. */
   GST_LIVE_LOCK (basesrc);
+  GST_DEBUG_OBJECT (basesrc, "unschedule clock");
 
   /* unblock clock sync (if any) */
   if (basesrc->clock_id)
     gst_clock_id_unschedule (basesrc->clock_id);
 
   /* configure what to do when we get to the LIVE lock. */
+  GST_DEBUG_OBJECT (basesrc, "live running %d", live_play);
   basesrc->live_running = live_play;
 
   if (live_play) {
@@ -2409,8 +2479,9 @@ gst_base_src_set_playing (GstBaseSrc * basesrc, gboolean live_play)
     if (start)
       gst_pad_start_task (basesrc->srcpad, (GstTaskFunction) gst_base_src_loop,
           basesrc->srcpad);
+    GST_DEBUG_OBJECT (basesrc, "signal");
+    GST_LIVE_SIGNAL (basesrc);
   }
-  GST_LIVE_SIGNAL (basesrc);
   GST_LIVE_UNLOCK (basesrc);
 
   return TRUE;
@@ -2558,6 +2629,7 @@ gst_base_src_change_state (GstElement * element, GstStateChange transition)
       no_preroll = gst_base_src_is_live (basesrc);
       break;
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
+      GST_DEBUG_OBJECT (basesrc, "PAUSED->PLAYING");
       if (gst_base_src_is_live (basesrc)) {
         /* now we can start playback */
         gst_base_src_set_playing (basesrc, TRUE);
@@ -2574,6 +2646,7 @@ gst_base_src_change_state (GstElement * element, GstStateChange transition)
 
   switch (transition) {
     case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
+      GST_DEBUG_OBJECT (basesrc, "PLAYING->PAUSED");
       if (gst_base_src_is_live (basesrc)) {
         /* make sure we block in the live lock in PAUSED */
         gst_base_src_set_playing (basesrc, FALSE);
@@ -2595,6 +2668,7 @@ gst_base_src_change_state (GstElement * element, GstStateChange transition)
         gst_pad_push_event (basesrc->srcpad, gst_event_new_eos ());
         basesrc->priv->last_sent_eos = TRUE;
       }
+      basesrc->priv->pending_eos = FALSE;
       event_p = &basesrc->data.ABI.pending_seek;
       gst_event_replace (event_p, NULL);
       event_p = &basesrc->priv->close_segment;
