@@ -234,6 +234,8 @@ struct _GstBaseTransformPrivate
   gboolean discont;
 
   GstActivateMode pad_mode;
+
+  gboolean gap_aware;
 };
 
 static GstElementClass *parent_class = NULL;
@@ -337,7 +339,7 @@ gst_base_transform_class_init (GstBaseTransformClass * klass)
       GST_DEBUG_FUNCPTR (gst_base_transform_get_property);
 
   g_object_class_install_property (gobject_class, PROP_QOS,
-      g_param_spec_boolean ("qos", "QoS", "handle QoS messages",
+      g_param_spec_boolean ("qos", "QoS", "Handle Quality-of-Service events",
           DEFAULT_PROP_QOS, G_PARAM_READWRITE));
 
   gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_base_transform_finalize);
@@ -400,6 +402,7 @@ gst_base_transform_init (GstBaseTransform * trans,
   trans->cache_caps1 = NULL;
   trans->cache_caps2 = NULL;
   trans->priv->pad_mode = GST_ACTIVATE_NONE;
+  trans->priv->gap_aware = FALSE;
 
   trans->passthrough = FALSE;
   if (bclass->transform == NULL) {
@@ -523,20 +526,21 @@ gst_base_transform_transform_size (GstBaseTransform * trans,
 no_in_size:
   {
     GST_DEBUG_OBJECT (trans, "could not get in_size");
-    g_warning ("could not get in_size");
+    g_warning ("%s: could not get in_size", GST_ELEMENT_NAME (trans));
     return FALSE;
   }
 no_multiple:
   {
     GST_DEBUG_OBJECT (trans, "Size %u is not a multiple of unit size %u", size,
         inunitsize);
-    g_warning ("Size %u is not a multiple of unit size %u", size, inunitsize);
+    g_warning ("%s: size %u is not a multiple of unit size %u",
+        GST_ELEMENT_NAME (trans), size, inunitsize);
     return FALSE;
   }
 no_out_size:
   {
     GST_DEBUG_OBJECT (trans, "could not get out_size");
-    g_warning ("could not get out_size");
+    g_warning ("%s: could not get out_size", GST_ELEMENT_NAME (trans));
     return FALSE;
   }
 }
@@ -974,6 +978,12 @@ gst_base_transform_prepare_output_buffer (GstBaseTransform * trans,
 
     gst_buffer_copy_metadata (*out_buf, in_buf,
         GST_BUFFER_COPY_FLAGS | GST_BUFFER_COPY_TIMESTAMPS);
+
+    /* Unset the GAP flag if the element is _not_ GAP aware. Otherwise
+     * it might create an output buffer that does not contain neutral data
+     * but still has the GAP flag on it! */
+    if (!trans->priv->gap_aware)
+      GST_BUFFER_FLAG_UNSET (*out_buf, GST_BUFFER_FLAG_GAP);
   }
 
 done:
@@ -1570,7 +1580,7 @@ gst_base_transform_chain (GstPad * pad, GstBuffer * buffer)
   GstClockTime last_stop = GST_CLOCK_TIME_NONE;
   GstBuffer *outbuf = NULL;
 
-  trans = GST_BASE_TRANSFORM (gst_pad_get_parent (pad));
+  trans = GST_BASE_TRANSFORM (GST_OBJECT_PARENT (pad));
 
   /* calculate end position of the incoming buffer */
   if (GST_BUFFER_TIMESTAMP (buffer) != GST_CLOCK_TIME_NONE) {
@@ -1613,8 +1623,6 @@ gst_base_transform_chain (GstPad * pad, GstBuffer * buffer)
     ret = GST_FLOW_OK;
   }
 
-  gst_object_unref (trans);
-
   return ret;
 }
 
@@ -1947,3 +1955,29 @@ gst_base_transform_is_qos_enabled (GstBaseTransform * trans)
 
   return result;
 }
+
+/**
+ * gst_base_transform_set_gap_aware:
+ * @trans: a #GstBaseTransform
+ * @gap_aware: New state
+ *
+ * If @gap_aware is %FALSE (as it is by default) subclasses will never get
+ * output buffers with the %GST_BUFFER_FLAG_GAP flag set.
+ *
+ * If set to %TRUE elements must handle output buffers with this flag set
+ * correctly, i.e. they can assume that the buffer contains neutral data
+ * but must unset the flag if the output is no neutral data.
+ * Since: 0.10.16
+ *
+ * MT safe.
+ */
+void
+gst_base_transform_set_gap_aware (GstBaseTransform * trans, gboolean gap_aware)
+{
+  g_return_if_fail (trans != NULL);
+
+  GST_OBJECT_LOCK (trans);
+  trans->priv->gap_aware = gap_aware;
+  GST_DEBUG_OBJECT (trans, "set gap aware %d", trans->priv->gap_aware);
+  GST_OBJECT_UNLOCK (trans);
+}
